"""
Tutorial:
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/animate

Usage:
    python3 examples/animate.py > examples/animate.svg
    chromium examples/animate.svg
"""
from datetime import timedelta
import svg


def draw() -> svg.SVG:
    return svg.SVG(
        x=0, y=0,
        width=100, height=100,
        elements=[svg.Rect(
            x=0, y=0,
            width=100, height=100,
            elements=[svg.Animate(
                attributeName="rx",
                values=[0, 50, 0],
                dur=timedelta(seconds=10),
                repeatCount="indefinite")]
        )]
    )


if __name__ == '__main__':
    print(draw())




from pathlib import Path

import pytest

import svg


FIXTURES = Path(__file__).parent / 'fixtures'
DEPRECATED = (FIXTURES / 'deprecated_elements.txt').read_text().split()


@pytest.mark.parametrize('cls', svg.Element.__subclasses__())
@pytest.mark.parametrize('name', DEPRECATED)
def test_no_deprecated__elements(cls: svg.Element, name: str):
    assert cls.element_name != name
    assert cls.element_name.lower() != name.lower()


@pytest.mark.parametrize('cls', svg.Element.__subclasses__())
def test_element_exported(cls: svg.Element):
    assert cls in vars(svg).values()



from pathlib import Path


POJECT_ROOT = Path(__file__).parent.parent.absolute()


def is_deprecated(path: Path) -> bool:
    if path.name == 'index.md':
        return False
    path /= 'index.md'
    if not path.is_file():
        raise LookupError(path)
    content = path.read_text()
    if '{{deprecated_header}}' in content.lower():
        return True
    if '- deprecated' in content:
        return True
    return False


def run() -> None:
    mdn_root = POJECT_ROOT.parent / 'mdn-source'
    svg_ref_root = mdn_root / 'files' / 'en-us' / 'web' / 'svg' / 'reference'

    out_path = POJECT_ROOT / 'tests' / 'fixtures' / 'deprecated_attrs.txt'
    with out_path.open('w') as out_stream:
        for ref_path in (svg_ref_root / 'attribute').iterdir():
            if is_deprecated(ref_path):
                print(ref_path.name, file=out_stream)

    # MDN doesn't list deprecated elements anymore.

    # out_path = POJECT_ROOT / 'tests' / 'fixtures' / 'deprecated_elements.txt'
    # with out_path.open('w') as out_stream:
    #     for ref_path in (svg_ref_root / 'element').iterdir():
    #         if is_deprecated(ref_path):
    #             print(ref_path.name, file=out_stream)


if __name__ == '__main__':
    run()



from __future__ import annotations

from ._types import Length, Number


def escape(text: str) -> str:
    """Make the text safe to use in SVG.
    """
    text = text.replace("&", "&amp;")
    text = text.replace(">", "&gt;")
    text = text.replace("<", "&lt;")
    return text


def mm(val: Number) -> Length:
    """Explicitly specify mm unit for the value.
    """
    return Length(value=val, unit='mm')


def px(val: Number) -> Length:
    """Explicitly specify px unit for the value.
    """
    return Length(value=val, unit='px')



"""
Tutorial:
    https://developer.mozilla.org/en-US/docs/Web/SVG/Tutorial/Basic_Shapes

Usage:
    python3 examples/shapes.py > examples/shapes.svg
    xdg-open examples/shapes.svg
"""
import svg


def draw() -> svg.SVG:
    return svg.SVG(
        width=200,
        height=250,
        elements=[
            svg.Rect(
                x=10, y=10,
                width=30, height=30,
                stroke="black",
                fill="transparent",
                stroke_width=5,
            ),
            svg.Rect(
                x=60, y=10,
                rx=10, ry=10,
                width=30, height=30,
                stroke="black",
                fill="transparent",
                stroke_width=5,
            ),
            svg.Circle(
                cx=25, cy=75, r=20,
                stroke="red",
                fill="transparent",
                stroke_width=5,
            ),
            svg.Ellipse(
                cx=75, cy=75,
                rx=20, ry=5,
                stroke="red",
                fill="transparent",
                stroke_width=5,
            ),
            svg.Line(
                x1=10, x2=50,
                y1=110, y2=150,
                stroke="orange",
                stroke_width=5,
            ),
            svg.Polyline(
                points=[
                    60, 110, 65, 120, 70, 115, 75, 130, 80,
                    125, 85, 140, 90, 135, 95, 150, 100, 145,
                ],
                stroke="orange",
                fill="transparent",
                stroke_width=5,
            ),
            svg.Polygon(
                points=[
                    50, 160, 55, 180, 70, 180, 60, 190, 65, 205,
                    50, 195, 35, 205, 40, 190, 30, 180, 45, 180,
                ],
                stroke="green",
                fill="transparent",
                stroke_width=5,
            ),
            svg.Path(
                d=[
                    svg.M(20, 230),
                    svg.Q(40, 205, 50, 230),
                    svg.T(90, 230),
                ],
                fill="none",
                stroke="blue",
                stroke_width=5,
            ),
        ],
    )


if __name__ == '__main__':
    print(draw())



"""SVG drawing library
"""
from ._filters import (
    FeBlend, FeColorMatrix, FeComponentTransfer, FeComposite, FeConvolveMatrix,
    FeDiffuseLighting, FeDisplacementMap, FeDistantLight, FeDropShadow,
    FeFlood, FeFuncA, FeFuncB, FeFuncG, FeFuncR, FeGaussianBlur, FeImage,
    FeMerge, FeMergeNode, FeMorphology, FeOffset, FePointLight,
    FeSpecularLighting, FeSpotLight, FeTile, FeTurbulence, Filter,
)
from ._helpers import escape, mm, px
from ._path import (
    Arc, ArcRel, C, ClosePath, CubicBezier, CubicBezierRel, H,
    HorizontalLineTo, HorizontalLineToRel, L, LineTo, LineToRel, M, MoveTo,
    MoveToRel, PathData, Q, QuadraticBezier, QuadraticBezierRel, S,
    SmoothCubicBezier, SmoothCubicBezierRel, SmoothQuadraticBezier,
    SmoothQuadraticBezierRel, T, V, VerticalLineTo, VerticalLineToRel, Z, a, c,
    h, l, m, q, s, t, v,
)
from ._transforms import (
    Matrix, Rotate, Scale, SkewX, SkewY, Transform, Translate,
)
from ._types import (
    Length, PreserveAspectRatio, ViewBoxSpec,
    TimeBezierPoint, SyncbaseValue, EventValue, RepeatValue, AccessKeyValue,
    AnimationTimingEvent,
)
from .elements import (
    SVG, A, Animate, AnimateMotion, AnimateTransform, Circle, ClipPath,
    ColorProfile, DefinitionSrc, Defs, Desc, Element, Ellipse, ForeignObject,
    G, Image, Line, LinearGradient, Marker, Mask, Metadata, MPath, Path,
    Pattern, Polygon, Polyline, RadialGradient, Rect, Script, Set, Stop, Style,
    Switch, Symbol, Text, TextPath, Title, TSpan, Use, View,
)

SemicolonSeparatedList = list
"""
DEPRECATED: we keep it only for backward compatibility. Use `list` instead.
"""


__version__ = '1.8.0'
__all__ = [
    'values',
    'escape',
    'mm',
    'px',

    # elements
    'Element',
    'A',
    'Animate',
    'AnimateMotion',
    'AnimateTransform',
    'Circle',
    'ClipPath',
    'ColorProfile',
    'DefinitionSrc',
    'Defs',
    'Desc',
    'Element',
    'Ellipse',
    'ForeignObject',
    'G',
    'Image',
    'Line',
    'LinearGradient',
    'Marker',
    'Mask',
    'Metadata',
    'MPath',
    'Path',
    'Pattern',
    'Polygon',
    'Polyline',
    'RadialGradient',
    'Rect',
    'Script',
    'Set',
    'Stop',
    'Style',
    'SVG',
    'Switch',
    'Symbol',
    'Text',
    'TextPath',
    'Title',
    'TSpan',
    'Use',
    'View',

    # filters
    'Filter',
    'FeBlend',
    'FeColorMatrix',
    'FeComponentTransfer',
    'FeComposite',
    'FeConvolveMatrix',
    'FeDiffuseLighting',
    'FeDisplacementMap',
    'FeDistantLight',
    'FeDropShadow',
    'FeFlood',
    'FeFuncA',
    'FeFuncB',
    'FeFuncG',
    'FeFuncR',
    'FeGaussianBlur',
    'FeImage',
    'FeMerge',
    'FeMergeNode',
    'FeMorphology',
    'FeOffset',
    'FePointLight',
    'FeSpecularLighting',
    'FeSpotLight',
    'FeTile',
    'FeTurbulence',

    # transforms
    'Transform',
    'Matrix',
    'Translate',
    'Scale',
    'Rotate',
    'SkewX',
    'SkewY',

    # path data
    'PathData',
    'M', 'MoveTo',
    'm', 'MoveToRel',
    'L', 'LineTo',
    'l', 'LineToRel',
    'H', 'HorizontalLineTo',
    'h', 'HorizontalLineToRel',
    'V', 'VerticalLineTo',
    'v', 'VerticalLineToRel',
    'C', 'CubicBezier',
    'c', 'CubicBezierRel',
    'S', 'SmoothCubicBezier',
    's', 'SmoothCubicBezierRel',
    'Q', 'QuadraticBezier',
    'q', 'QuadraticBezierRel',
    'T', 'SmoothQuadraticBezier',
    't', 'SmoothQuadraticBezierRel',
    'A', 'Arc',
    'a', 'ArcRel',
    'Z', 'ClosePath',

    # types
    'AccessKeyValue',
    'AnimationTimingEvent',
    'EventValue',
    'Length',
    'PreserveAspectRatio',
    'RepeatValue',
    'SyncbaseValue',
    'TimeBezierPoint',
    'ViewBoxSpec',
]



from pathlib import Path

import pytest

import reflect


LIB_ELEMENTS = reflect.LibElement.parse_all()
ROOT = Path(__file__).parent.parent.parent
MDN_ROOT = ROOT / 'mdn-source' / 'files' / 'en-us' / 'web' / 'svg' / 'reference'
# If the line below fails, run the following one directory up the root of svg.py:
#   git clone https://github.com/mdn/content.git mdn-source --depth 1
assert MDN_ROOT.is_dir(), f'you must clone mdn-source into {MDN_ROOT} before you can run tests'
MDN_ATTRS = reflect.MDNAttr.parse_all(MDN_ROOT)

FIXTURES = Path(__file__).parent / 'fixtures'
DEPRECATED = (FIXTURES / 'deprecated_attrs.txt').read_text().split()

assert LIB_ELEMENTS
assert MDN_ATTRS


@pytest.mark.parametrize('lib_element', LIB_ELEMENTS)
@pytest.mark.parametrize('mdn_attr', MDN_ATTRS)
def test_attrs(lib_element: reflect.LibElement, mdn_attr: reflect.MDNAttr):
    if mdn_attr.is_deprecated:
        msg = f'attr `{mdn_attr.title}` is deprecated but is set for `{lib_element.title}`'
        assert mdn_attr.title not in lib_element.attrs, msg
        return
    if mdn_attr.title in DEPRECATED:
        msg = f'attr `{mdn_attr.title}` is deprecated but is set for `{lib_element.title}`'
        assert mdn_attr.title not in lib_element.attrs, msg
        return

    if lib_element.title in mdn_attr.elements:
        msg = f'attr `{mdn_attr.title}` should be set for `{lib_element.title}` but it is not'
        assert mdn_attr.title in lib_element.attrs, msg

    # <g> and <use> may have additional undocumented attributes
    if lib_element.title in {'g', 'use'}:
        return
    if mdn_attr.elements and lib_element.title not in mdn_attr.elements:
        msg = f'attr `{mdn_attr.title}` should not be set for `{lib_element.title}` but it is'
        assert mdn_attr.title not in lib_element.attrs, msg



from __future__ import annotations

import svg


STROKE_LEN = 10
GAP = 10
WIDTH = 400
HEIGHT = 400

SHIFT = int(STROKE_LEN / 2)
STEP = STROKE_LEN + GAP


def draw() -> svg.SVG:
    elements: list[svg.Element] = []
    do_shift = True
    for y in range(STROKE_LEN, HEIGHT, STEP):
        do_shift = not do_shift
        start_x = STROKE_LEN
        if do_shift:
            start_x += STROKE_LEN
        for x in range(start_x, WIDTH, STEP):
            for angle in (0, 60, -60):
                elements.append(svg.Path(
                    stroke="#2c3e50",
                    stroke_width=1,
                    stroke_linecap="round",
                    transform=[svg.Rotate(angle, x, y)],
                    d=[
                        svg.M(x - SHIFT, y),
                        svg.L(x + SHIFT, y),
                    ],
                ))
    return svg.SVG(
        viewBox=svg.ViewBoxSpec(0, 0, WIDTH, HEIGHT),
        elements=elements,
    )


if __name__ == '__main__':
    print(draw())



"""
Tutorial:
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/animateTransform

Usage:
    python3 examples/animate_transform.py > examples/animate_transform.svg
    chromium examples/animate_transform.svg
"""
from datetime import timedelta
import svg


def draw() -> svg.SVG:
    return svg.SVG(
        x=0, y=0,
        width=120, height=120,
        elements=[svg.Polygon(
            points=[60, 30, 90, 90, 30, 90],
            elements=[svg.AnimateTransform(
                attributeName="transform",
                type="rotate",
                from_="0 60 70",
                to="360 60 70",
                dur=timedelta(seconds=10),
                repeatCount="indefinite",
            )]
        )]
    )


if __name__ == '__main__':
    print(draw())



from __future__ import annotations

from dataclasses import dataclass
from enum import Enum
from typing import TYPE_CHECKING, Any, ClassVar
from datetime import timedelta, datetime

from . import _mixins as m
from ._path import PathData
from ._transforms import Transform
from ._types import Length, Number, PreserveAspectRatio, ViewBoxSpec, to_clock_value, to_wallclock_sync_value


if TYPE_CHECKING:
    from typing_extensions import Literal


# Attributes whose elements should be separated by a semicolon (;) rather than a space.
_SEMICOLON_ATTRS = frozenset({
    "values",
    "keyTimes",
    "keySplines",
    "keyPoints",
    "begin",
    "end",
})
_SEMICOLON_TYPES = frozenset({
    m.Animation,
    m.AnimationTiming,
})


@dataclass
class Element:
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/Core
    """

    element_name: ClassVar[str]

    elements: list[Element] | None = None
    text: str | None = None
    id: str | None = None
    tabindex: int | None = None
    lang: str | None = None

    transform_origin: str | None = None
    style: str | None = None
    data: dict | None = None

    extra: dict[str, str] | None = None
    """Any extra attributes to add to the element.

    Useful for HTMX and other HTML frameworks. Use with caution.
    """

    @classmethod
    def _as_str(cls, val: Any, key: str | None = None) -> str:
        if val is None:
            return ""
        if isinstance(val, Element):
            return str(val)
        if isinstance(val, Enum):
            return val.value
        if isinstance(val, bool):
            return str(val).lower()
        if isinstance(val, (list, tuple)):
            sep = " "
            # Some attributes of some animation-related elements
            # use semicolon instead of space to separate list elements.
            if key in _SEMICOLON_ATTRS:
                if set(cls.__bases__) & _SEMICOLON_TYPES:
                    sep = ";"
            return sep.join(cls._as_str(v) for v in val)
        if isinstance(val, timedelta):
            return to_clock_value(val)
        if isinstance(val, datetime):
            return to_wallclock_sync_value(val)
        return str(val)

    def as_dict(self) -> dict[str, str]:
        result = {}
        for key, val in vars(self).items():
            if val is None:
                continue
            if key in ("elements", "text", "data", "attributes"):
                continue
            key = key.rstrip("_")
            key = key.replace("__", ":")
            key = key.replace("_", "-")
            result[key] = self._as_str(val, key=key)
        return result

    def as_str(self) -> str:
        props = " ".join(f'{k}="{v}"' for k, v in self.as_dict().items())
        if self.data:
            if props:
                props += " "
            props += " ".join(f'data-{k}="{v}"' for k, v in self.data.items())
        if self.extra:
            if props:
                props += " "
            props += " ".join(f'{k}="{v}"' for k, v in self.extra.items())
        if props:
            props = " " + props
        if self.text:
            return f"<{self.element_name}{props}>{self.text}</{self.element_name}>"
        if self.elements:
            content = "".join(self._as_str(e) for e in self.elements)
            return f"<{self.element_name}{props}>{content}</{self.element_name}>"
        return f"<{self.element_name}{props}/>"

    def __str__(self) -> str:
        return self.as_str()


@dataclass
class SVG(
    Element,
    m.GraphicsElementEvents,
    m.Color,
    m.Graphics,
):
    """The svg element is a container that defines a new coordinate system and viewport.

    It is used as the outermost element of SVG documents, but it can also be used
    to embed an SVG fragment inside an SVG or HTML document.

    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/svg
    """
    element_name = "svg"
    xmlns: str | None = "http://www.w3.org/2000/svg"
    viewBox: ViewBoxSpec | None = None
    preserveAspectRatio: PreserveAspectRatio | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    class_: list[str] | None = None
    mask: str | None = None
    opacity: Number | None = None
    clip_path: str | None = None
    overflow: Literal["visible", "hidden", "scroll", "auto", "inherit"] | None = None

    onunload: str | None = None
    onabort: str | None = None
    onerror: str | None = None
    onresize: str | None = None
    onscroll: str | None = None
    onzoom: str | None = None


@dataclass
class G(
    Element,
    m.GraphicsElementEvents,
    m.Color,
    m.Graphics,
    m.FillStroke,
):
    """The <g> SVG element is a container used to group other SVG elements.

    Transformations applied to the <g> element are performed on its child elements,
    and its attributes are inherited by its children. It can also group multiple elements
    to be referenced later with the <use> element.

    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/g
    """
    element_name = "g"
    transform: list[Transform] | None = None
    class_: list[str] | None = None
    mask: str | None = None
    opacity: Number | None = None
    clip_path: str | None = None
    fill_rule: Literal["evenodd", "nonzero", "inherit"] | None = None
    fill_opacity: Number | None = None
    fill: str | None = None


@dataclass
class Defs(
    Element,
    m.Color,
    m.GraphicsElementEvents,
):
    """The <defs> is used to store graphical objects that will be used at a later time.

    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/defs
    """
    element_name = "defs"
    transform: list[Transform] | None = None
    class_: list[str] | None = None
    pointer_events: str | None = None  # TODO


@dataclass
class Desc(Element, m.GraphicsElementEvents):
    """The <desc> element provides an accessible, long-text description of any element.

    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/desc
    """
    element_name = "desc"
    content: str | None = None
    class_: list[str] | None = None


@dataclass
class Title(Element, m.GraphicsElementEvents):
    """The <title> element provides an accessible, short-text description of any element.

    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/title
    """
    element_name = "title"
    content: str | None = None
    class_: list[str] | None = None


@dataclass
class Symbol(
    Element,
    m.GraphicsElementEvents,
    m.Color,
    m.Graphics,
):
    """The <symbol> is used to define template objects which can be used by a <use> element.

    The use of <symbol> elements for graphics that are used multiple times in the same
    document adds structure and semantics. Documents that are rich in structure may be
    rendered graphically, as speech, or as Braille, and thus promote accessibility.

    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/symbol
    """
    element_name = "symbol"
    viewBox: ViewBoxSpec | None = None
    preserveAspectRatio: PreserveAspectRatio | None = None
    refX: Length | Number | None = None
    refY: Length | Number | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    class_: list[str] | None = None
    mask: str | None = None
    opacity: Number | None = None
    clip_path: str | None = None
    overflow: Literal["visible", "hidden", "scroll", "auto", "inherit"] | None = None


@dataclass
class Image(
    Element,
    m.Color,
    m.Graphics,
    m.GraphicsElementEvents,
):
    """The <image> SVG element includes images inside SVG documents.

    The only image formats SVG software must support are JPEG, PNG, and other SVG files.
    Animated GIF behavior is undefined.

    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/image
    """
    element_name = "image"
    href: str | None = None
    transform: list[Transform] | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    preserveAspectRatio: PreserveAspectRatio | None = None
    image_rendering: Literal["auto", "optimizeSpeed", "optimizeQuality"] | None = None
    class_: list[str] | None = None
    vector_effect: Literal["none", "non-scaling-stroke", "non-scaling-size", "non-rotation", "fixed-position"] | None = None
    visibility: Literal["visible", "hidden", "inherit"] | None = None
    mask: str | None = None
    opacity: Number | None = None
    clip_path: str | None = None
    overflow: Literal["visible", "hidden", "scroll", "auto", "inherit"] | None = None


@dataclass
class Switch(Element, m.Color, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/switch
    """
    element_name = "switch"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    opacity: Number | None = None
    class_: list[str] | None = None
    pointer_events: str | None = None  # TODO


@dataclass
class Style(Element, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/style
    """
    element_name = "style"
    type: str | None = None
    media: list[str] | None = None
    title: str | None = None


@dataclass
class _FigureElement(m.Color, m.GraphicsElementEvents, m.Graphics, m.FillStroke):
    pathLength: float | None = None
    paint_order: Literal["normal", "fill", "stroke", "markers"] | None = None
    shape_rendering: Literal["auto", "optimizeSpeed", "crispEdges", "geometricPrecision", "inherit"] | None = None
    class_: list[str] | None = None
    vector_effect: Literal["none", "non-scaling-stroke", "non-scaling-size", "non-rotation", "fixed-position"] | None = None
    visibility: Literal["visible", "hidden", "inherit"] | None = None
    mask: str | None = None
    opacity: Number | None = None
    clip_path: str | None = None


@dataclass
class Path(Element, _FigureElement):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/path
    """
    element_name = "path"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    d: list[PathData] | None = None
    marker_start: str | None = None
    marker_mid: str | None = None
    marker_end: str | None = None
    stroke_linecap: Literal["butt", "round", "square", "inherit"] | None = None
    stroke_linejoin: Literal["miter", "round", "bevel", "inherit"] | None = None
    stroke_miterlimit: Number | None = None
    fill_rule: Literal["evenodd", "nonzero", "inherit"] | None = None
    fill_opacity: Number | None = None
    fill: str | None = None


@dataclass
class Rect(Element, _FigureElement):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/rect
    """
    element_name = "rect"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    rx: Length | Number | None = None
    ry: Length | Number | None = None
    marker_start: str | None = None
    marker_mid: str | None = None
    marker_end: str | None = None
    stroke_linejoin: Literal["miter", "round", "bevel", "inherit"] | None = None
    stroke_miterlimit: Number | None = None
    fill_opacity: Number | None = None
    fill: str | None = None


@dataclass
class Circle(Element, _FigureElement):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/circle
    """
    element_name = "circle"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    cx: Length | Number | None = None
    cy: Length | Number | None = None
    r: Length | Number | None = None
    marker_mid: str | None = None
    fill_opacity: Number | None = None
    fill: str | None = None


@dataclass
class Ellipse(Element, _FigureElement):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/ellipse
    """
    element_name = "ellipse"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    cx: Length | Number | None = None
    cy: Length | Number | None = None
    rx: Length | Number | None = None
    ry: Length | Number | None = None
    marker_start: str | None = None
    marker_mid: str | None = None
    marker_end: str | None = None
    fill_opacity: Number | None = None
    fill: str | None = None


@dataclass
class Line(Element, _FigureElement):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/line
    """
    element_name = "line"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    x1: Length | Number | None = None
    y1: Length | Number | None = None
    x2: Length | Number | None = None
    y2: Length | Number | None = None
    marker_start: str | None = None
    marker_mid: str | None = None
    marker_end: str | None = None
    stroke_linecap: Literal["butt", "round", "square", "inherit"] | None = None


@dataclass
class Polyline(Element, _FigureElement):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/polyline
    """
    element_name = "polyline"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    points: list[Number] | None = None
    marker_start: str | None = None
    marker_mid: str | None = None
    marker_end: str | None = None
    stroke_linecap: Literal["butt", "round", "square", "inherit"] | None = None
    stroke_linejoin: Literal["miter", "round", "bevel", "inherit"] | None = None
    stroke_miterlimit: Number | None = None
    fill_rule: Literal["evenodd", "nonzero", "inherit"] | None = None
    fill_opacity: Number | None = None
    fill: str | None = None


@dataclass
class Polygon(Element, _FigureElement):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/polygon
    """
    element_name = "polygon"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    points: list[Number] | None = None
    marker_start: str | None = None
    marker_mid: str | None = None
    marker_end: str | None = None
    stroke_linejoin: Literal["miter", "round", "bevel", "inherit"] | None = None
    stroke_miterlimit: Number | None = None
    fill_rule: Literal["evenodd", "nonzero", "inherit"] | None = None
    fill_opacity: Number | None = None
    fill: str | None = None


@dataclass
class _TextElement(
    m.FontSpecification,
    m.TextContentElements,
    m.Color,
    m.GraphicsElementEvents,
    m.Graphics,
    m.FillStroke,
):
    paint_order: Literal["normal", "fill", "stroke", "markers"] | None = None
    class_: list[str] | None = None
    vector_effect: Literal["none", "non-scaling-stroke", "non-scaling-size", "non-rotation", "fixed-position"] | None = None
    visibility: Literal["visible", "hidden", "inherit"] | None = None
    fill_opacity: Number | None = None
    fill: str | None = None


@dataclass
class Text(Element, _TextElement):
    """The SVG <text> element draws a graphics element consisting of text.

    It's possible to apply a gradient, pattern, clipping path, mask, or filter to <text>,
    like any other SVG graphics element.

    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/text
    """
    element_name = "text"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    dx: Length | Number | None = None
    dy: Length | Number | None = None
    textLength: Length | Number | None = None
    lengthAdjust: Literal["spacing", "spacingAndGlyphs"] | None = None
    writing_mode: Literal["horizontal-tb", "vertical-rl", "vertical-lr"] | None = None
    text_rendering: Literal["auto", "optimizeSpeed", "optimizeLegibility", "geometricPrecision"] | None = None
    stroke_linecap: Literal["butt", "round", "square", "inherit"] | None = None
    stroke_linejoin: Literal["miter", "round", "bevel", "inherit"] | None = None
    stroke_miterlimit: Number | None = None
    fill_rule: Literal["evenodd", "nonzero", "inherit"] | None = None
    mask: str | None = None
    opacity: Number | None = None
    clip_path: str | None = None
    overflow: Literal["visible", "hidden", "scroll", "auto", "inherit"] | None = None


@dataclass
class TSpan(Element, _TextElement):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/tspan
    """
    element_name = "tspan"
    externalResourcesRequired: bool | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    dx: Length | Number | None = None
    dy: Length | Number | None = None
    textLength: Length | Number | None = None
    lengthAdjust: Literal["spacing", "spacingAndGlyphs"] | None = None
    writing_mode: Literal["horizontal-tb", "vertical-rl", "vertical-lr"] | None = None
    alignment_baseline: None | Literal[
        "baseline", "top", "before-edge", "text-top",
        "text-before-edge", "middle", "bottom",
        "after-edge", "text-bottom", "text-after-edge", "ideographic",
        "lower", "hanging", "mathematical", "inherit",
    ] = None
    baseline_shift: Literal["baseline", "sub", "super", "inherit"] | None = None
    stroke_linecap: Literal["butt", "round", "square", "inherit"] | None = None
    stroke_linejoin: Literal["miter", "round", "bevel", "inherit"] | None = None
    stroke_miterlimit: Number | None = None
    fill_rule: Literal["evenodd", "nonzero", "inherit"] | None = None
    opacity: Number | None = None


@dataclass
class TextPath(Element, _TextElement):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/textPath
    """
    element_name = "textPath"
    externalResourcesRequired: bool | None = None
    startOffset: str | None = None
    textLength: Length | Number | None = None
    lengthAdjust: Literal["spacing", "spacingAndGlyphs"] | None = None
    method: Literal["align", "stretch"] | None = None
    spacing: Literal["auto", "exact"] | None = None
    href: str | None = None
    path: str | None = None
    side: Literal["left", "right"] | None = None
    writing_mode: Literal["horizontal-tb", "vertical-rl", "vertical-lr"] | None = None
    alignment_baseline: None | Literal[
        "baseline", "top", "before-edge", "text-top",
        "text-before-edge", "middle", "bottom",
        "after-edge", "text-bottom", "text-after-edge", "ideographic",
        "lower", "hanging", "mathematical", "inherit",
    ] = None
    baseline_shift: Literal["baseline", "sub", "super", "inherit"] | None = None
    stroke_linecap: Literal["butt", "round", "square", "inherit"] | None = None
    stroke_linejoin: Literal["miter", "round", "bevel", "inherit"] | None = None
    stroke_miterlimit: Number | None = None
    fill_rule: Literal["evenodd", "nonzero", "inherit"] | None = None
    opacity: Number | None = None


@dataclass
class Marker(Element, m.Color, m.GraphicsElementEvents, m.Graphics):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/marker
    """
    element_name = "marker"
    externalResourcesRequired: bool | None = None
    viewBox: ViewBoxSpec | None = None
    preserveAspectRatio: PreserveAspectRatio | None = None
    refX: Length | Number | None = None
    refY: Length | Number | None = None
    markerUnits: Literal["strokeWidth", "userSpaceOnUse", "userSpace"] | None = None
    markerWidth: Length | Number | None = None
    markerHeight: Length | Number | None = None
    orient: str | None = None
    opacity: Number | None = None
    clip_path: str | None = None
    class_: list[str] | None = None
    mask: str | None = None
    overflow: Literal["visible", "hidden", "scroll", "auto", "inherit"] | None = None


@dataclass
class ColorProfile(Element):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/color
    """
    element_name = "color-profile"
    local: str | None = None


@dataclass
class _Gradient:
    externalResourcesRequired: bool | None = None
    gradientUnits: Literal["userSpaceOnUse", "objectBoundingBox"] | None = None
    gradientTransform: list[Transform] | None = None
    spreadMethod: Literal["pad", "reflect", "repeat"] | None = None
    href: str | None = None
    class_: list[str] | None = None


@dataclass
class LinearGradient(Element, _Gradient, m.Color, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/linearGradient
    """
    element_name = "linearGradient"
    x1: Length | Number | None = None
    y1: Length | Number | None = None
    x2: Length | Number | None = None
    y2: Length | Number | None = None


@dataclass
class RadialGradient(Element, _Gradient, m.Color, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/radialGradient
    """
    element_name = "radialGradient"
    cx: Length | Number | None = None
    cy: Length | Number | None = None
    r: Length | Number | None = None
    fr: Length | Number | None = None
    fx: Length | Number | None = None
    fy: Length | Number | None = None


@dataclass
class Stop(Element, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/stop
    """
    element_name = "stop"
    offset: Length | Number | None = None
    stop_opacity: Number | None = None
    stop_color: str | None = None
    class_: list[str] | None = None


@dataclass
class Pattern(Element, m.Color, m.GraphicsElementEvents, m.Graphics):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/pattern
    """
    element_name = "pattern"
    externalResourcesRequired: bool | None = None
    viewBox: ViewBoxSpec | None = None
    preserveAspectRatio: PreserveAspectRatio | None = None
    patternUnits: Literal["userSpaceOnUse", "objectBoundingBox"] | None = None
    patternTransform: list[Transform] | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    patternContentUnits: Literal["userSpaceOnUse", "objectBoundingBox"] | None = None
    href: str | None = None
    class_: list[str] | None = None
    mask: str | None = None
    clip_path: str | None = None
    overflow: Literal["visible", "hidden", "scroll", "auto", "inherit"] | None = None


@dataclass
class ClipPath(Element, m.Color, m.Graphics):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/clipPath
    """
    element_name = "clipPath"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    clipPathUnits: Literal["userSpaceOnUse", "objectBoundingBox"] | None = None
    class_: list[str] | None = None
    mask: str | None = None
    clip_path: str | None = None


@dataclass
class Mask(Element, m.Color, m.Graphics):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/mask
    """
    element_name = "mask"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    maskUnits: Literal["userSpaceOnUse", "objectBoundingBox"] | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    maskContentUnits: Literal["userSpaceOnUse", "objectBoundingBox"] | None = None
    class_: list[str] | None = None
    mask: str | None = None
    clip_path: str | None = None


@dataclass
class A(Element, m.Color, m.GraphicsElementEvents, m.Graphics):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/a
    """
    element_name = "a"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    target: Literal["_self", "_parent", "_top", "_blank"] | None = None
    href: str | None = None
    class_: list[str] | None = None
    visibility: Literal["visible", "hidden", "inherit"] | None = None
    mask: str | None = None
    opacity: Number | None = None
    clip_path: str | None = None


@dataclass
class View(Element, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/view
    """
    element_name = "view"
    externalResourcesRequired: bool | None = None
    viewBox: ViewBoxSpec | None = None
    preserveAspectRatio: PreserveAspectRatio | None = None


@dataclass
class Script(Element, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/script
    """
    element_name = "script"
    externalResourcesRequired: bool | None = None
    type: str | None = None
    href: str | None = None


@dataclass
class Animate(Element, m.Animation, m.Color, m.AnimationTiming, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/animate
    """
    element_name = "animate"
    externalResourcesRequired: bool | None = None
    attributeName: str | None = None


@dataclass
class Set(Element, m.AnimationTiming, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/set
    """
    element_name = "set"
    externalResourcesRequired: bool | None = None
    to: str | None = None
    attributeName: str | None = None
    href: str | None = None


@dataclass
class AnimateMotion(Element, m.Animation, m.AnimationTiming, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/animateMotion
    """
    element_name = "animateMotion"
    externalResourcesRequired: bool | None = None
    path: list[PathData] | None = None
    rotate: Number | Literal["auto", "auto-reverse"] | None = None
    origin: Literal["default"] | None = None


@dataclass
class MPath(Element, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/mpath
    """
    element_name = "mpath"
    externalResourcesRequired: bool | None = None
    href: str | None = None


@dataclass
class AnimateTransform(Element, m.Animation, m.AnimationTiming, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/animateTransform
    """
    element_name = "animateTransform"
    externalResourcesRequired: bool | None = None
    type: Literal["translate", "scale", "rotate", "skewX", "skewY"] | None = None
    attributeName: str | None = None


@dataclass
class DefinitionSrc(Element):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/definition
    """
    element_name = "definition-src"
    pass


@dataclass
class Metadata(Element, m.GraphicsElementEvents):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/metadata
    """
    element_name = "metadata"
    pass


@dataclass
class ForeignObject(Element, m.Color, m.GraphicsElementEvents, m.Graphics):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/foreignObject
    """
    element_name = "foreignObject"
    externalResourcesRequired: bool | None = None
    transform: list[Transform] | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    content: str | None = None
    class_: list[str] | None = None
    vector_effect: Literal["none", "non-scaling-stroke", "non-scaling-size", "non-rotation", "fixed-position"] | None = None
    visibility: Literal["visible", "hidden", "inherit"] | None = None
    opacity: Number | None = None
    overflow: Literal["visible", "hidden", "scroll", "auto", "inherit"] | None = None


@dataclass
class Use(Element, m.GraphicsElementEvents, m.Color, m.Graphics, m.FillStroke):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/use
    """
    element_name = "use"
    href: str | None = None
    class_: list[str] | None = None
    style: str | None = None
    transform: list[Transform] | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    vector_effect: Literal["none", "non-scaling-stroke", "non-scaling-size", "non-rotation", "fixed-position"] | None = None
    opacity: Number | None = None
    clip_path: str | None = None
    mask: str | None = None
    fill_rule: Literal["evenodd", "nonzero", "inherit"] | None = None
    fill_opacity: Number | None = None
    fill: str | None = None



from datetime import datetime, timedelta

import pytest

from svg._types import to_clock_value, to_wallclock_sync_value


@pytest.mark.parametrize("input, expected", [
    # Timecount-val
    (timedelta(), "0s"),
    (timedelta(seconds=1), "1s"),
    (timedelta(seconds=-1), "-1s"),
    (timedelta(seconds=10), "10s"),
    (timedelta(seconds=-10), "-10s"),
    (timedelta(seconds=0.5), "0.5s"),
    (timedelta(seconds=-0.5), "-0.5s"),
    (timedelta(seconds=0.002), "0.002s"),
    (timedelta(seconds=-0.002), "-0.002s"),
    (timedelta(milliseconds=1), "0.001s"),
    (timedelta(milliseconds=-1), "-0.001s"),

    # Full-clock-val
    (timedelta(seconds=70), "0:01:10"),
    (timedelta(seconds=605), "0:10:05"),
    (timedelta(days=10), "240:00:00"),
    (timedelta(hours=5, minutes=7, seconds=12.2), "5:07:12.2"),
    (timedelta(hours=-1), "-1:00:00"),
])
def test_to_clock_value(input, expected):
    actual = to_clock_value(input)
    assert actual == expected


@pytest.mark.parametrize("input, expected", [
    (
        datetime(year=2025, month=7, day=9),
        "wallclock(2025-07-09 00:00:00)",
    ),
    (
        datetime(year=1999, month=1, day=1, hour=13, minute=37, second=12, microsecond=4000),
        "wallclock(1999-01-01 13:37:12.004000)",
    ),
])
def test_to_wallclock_sync_value(input, expected):
    actual = to_wallclock_sync_value(input)
    assert actual == expected



import svg


STROKE_LEN = 10
GAP = 10
WIDTH = 400
HEIGHT = 400

SHIFT = int(STROKE_LEN / 2)
STEP = STROKE_LEN + GAP


def draw() -> svg.SVG:
    lines = []
    for x in range(STROKE_LEN, WIDTH, STEP):
        for y in range(STROKE_LEN, HEIGHT, STEP):
            lines.extend([
                # horizontal line
                svg.M(x - SHIFT, y),
                svg.L(x + SHIFT, y),
                # vertical line
                svg.M(x, y - SHIFT),
                svg.L(x, y + SHIFT),
            ])
    return svg.SVG(
        viewBox=svg.ViewBoxSpec(0, 0, WIDTH, HEIGHT),
        elements=[svg.Path(
            stroke="#2c3e50",
            stroke_width=1,
            stroke_linecap="round",
            d=lines,
        )],
    )


if __name__ == '__main__':
    print(draw())



from __future__ import annotations

from dataclasses import astuple, dataclass
from typing import ClassVar

from ._types import Number


@dataclass
class PathData:
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d
    """
    command: ClassVar[str]

    def __str__(self) -> str:
        points = []
        for p in astuple(self):
            if isinstance(p, bool):
                p = int(p)
            points.append(str(p))
        joined = " ".join(points)
        return f"{self.command} {joined}"


@dataclass
class MoveTo(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#moveto_path_commands
    """
    command = 'M'
    x: Number
    y: Number


@dataclass
class MoveToRel(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#moveto_path_commands
    """
    command = 'm'
    dx: Number
    dy: Number


@dataclass
class LineTo(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#lineto_path_commands
    """
    command = 'L'
    x: Number
    y: Number


@dataclass
class LineToRel(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#lineto_path_commands
    """
    command = 'l'
    dx: Number
    dy: Number


@dataclass
class HorizontalLineTo(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#lineto_path_commands
    """
    command = 'H'
    x: Number


@dataclass
class HorizontalLineToRel(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#lineto_path_commands
    """
    command = 'h'
    dx: Number


@dataclass
class VerticalLineTo(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#lineto_path_commands
    """
    command = 'V'
    y: Number


@dataclass
class VerticalLineToRel(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#lineto_path_commands
    """
    command = 'v'
    dy: Number


@dataclass
class CubicBezier(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#cubic_b%C3%A9zier_curve
    """
    command = 'C'
    x1: Number
    y1: Number
    x2: Number
    y2: Number
    x: Number
    y: Number


@dataclass
class CubicBezierRel(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#cubic_b%C3%A9zier_curve
    """
    command = 'c'
    dx1: Number
    dy1: Number
    dx2: Number
    dy2: Number
    dx: Number
    dy: Number


@dataclass
class SmoothCubicBezier(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#cubic_b%C3%A9zier_curve
    """
    command = 'S'
    x2: Number
    y2: Number
    x: Number
    y: Number


@dataclass
class SmoothCubicBezierRel(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#cubic_b%C3%A9zier_curve
    """
    command = 's'
    dx2: Number
    dy2: Number
    dx: Number
    dy: Number


@dataclass
class QuadraticBezier(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#quadratic_b%C3%A9zier_curve
    """
    command = 'Q'
    x1: Number
    y1: Number
    x: Number
    y: Number


@dataclass
class QuadraticBezierRel(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#quadratic_b%C3%A9zier_curve
    """
    command = 'q'
    dx1: Number
    dy1: Number
    dx: Number
    dy: Number


@dataclass
class SmoothQuadraticBezier(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#quadratic_b%C3%A9zier_curve
    """
    command = 'T'
    x: Number
    y: Number


@dataclass
class SmoothQuadraticBezierRel(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#quadratic_b%C3%A9zier_curve
    """
    command = 't'
    dx: Number
    dy: Number


@dataclass
class Arc(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#elliptical_arc_curve
    """
    command = 'A'
    rx: Number
    ry: Number
    angle: Number
    large_arc: bool
    sweep: bool
    x: Number
    y: Number


@dataclass
class ArcRel(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#elliptical_arc_curve
    """
    command = 'a'
    rx: Number
    ry: Number
    angle: Number
    large_arc: bool
    sweep: bool
    dx: Number
    dy: Number


@dataclass
class ClosePath(PathData):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/d#closepath
    """
    command = 'Z'


# aliases
M = MoveTo
m = MoveToRel
L = LineTo
l = LineToRel  # noqa: E741
H = HorizontalLineTo
h = HorizontalLineToRel
V = VerticalLineTo
v = VerticalLineToRel
C = CubicBezier
c = CubicBezierRel
S = SmoothCubicBezier
s = SmoothCubicBezierRel
Q = QuadraticBezier
q = QuadraticBezierRel
T = SmoothQuadraticBezier
t = SmoothQuadraticBezierRel
A = Arc
a = ArcRel
Z = ClosePath



"""
Tutorial:
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/text

Usage:
    python3 examples/text.py > examples/text.svg
    chromium examples/text.svg
"""
from textwrap import dedent

import svg


def draw() -> svg.SVG:
    return svg.SVG(
        width=240,
        height=80,
        elements=[
            svg.Style(
                text=dedent("""
                    .small { font: italic 13px sans-serif; }
                    .heavy { font: bold 30px sans-serif; }

                    /* Note that the color of the text is set with the    *
                    * fill property, the color property is for HTML only */
                    .Rrrrr { font: italic 40px serif; fill: red; }
                """),
            ),
            svg.Text(x=20, y=35, class_=["small"], text="My"),
            svg.Text(x=40, y=35, class_=["heavy"], text="cat"),
            svg.Text(x=55, y=55, class_=["small"], text="is"),
            svg.Text(x=65, y=55, class_=["Rrrrr"], text="Grumpy!"),
        ],
    )


if __name__ == '__main__':
    print(draw())



from pathlib import Path

import pytest

import examples
import svg


@pytest.mark.parametrize('name', examples.__all__)
def test_examples_up_to_date(name: str) -> None:
    module = getattr(examples, name)
    root_element = module.draw()
    assert isinstance(root_element, svg.SVG)

    out_path = Path(__file__).parent.parent / 'examples' / f'{name}.svg'
    expected = out_path.read_text()
    assert str(root_element) == expected.strip()



from __future__ import annotations

from dataclasses import dataclass
from datetime import timedelta
from typing import TYPE_CHECKING, Any

from ._types import AnimationTimingEvent, Length, Number, TimeBezierPoint


if TYPE_CHECKING:
    from typing_extensions import Literal


class AttrsMixin:
    pass


@dataclass
class GraphicsElementEvents(AttrsMixin):
    onfocusin: str | None = None
    onfocusout: str | None = None
    onactivate: str | None = None
    onclick: str | None = None
    onmousedown: str | None = None
    onmouseup: str | None = None
    onmouseover: str | None = None
    onmousemove: str | None = None
    onmouseout: str | None = None
    onload: str | None = None


@dataclass
class Color(AttrsMixin):
    color: str | None = None
    color_interpolation: None | Literal[
        "auto", "sRGB", "linearRGB", "inherit",
    ] = None


@dataclass
class FillStroke(AttrsMixin):
    stroke: str | None = None
    stroke_dasharray: list[Number] | Literal["none"] | Length | None = None
    stroke_dashoffset: Literal["none"] | Length | Number | None = None
    stroke_opacity: Number | None = None
    stroke_width: Length | Number | None = None


@dataclass
class FontSpecification(AttrsMixin):
    font_family: str | None = None
    font_size: Length | Number | None = None
    font_size_adjust: Number | None | Literal["none"] = None
    font_style: None | Literal[
        "normal", "italic", "oblique", "inherit",
    ] = None
    font_variant: Literal["normal", "small-caps", "inherit"] | None = None
    font_weight: None | Literal[
        "normal", "bold", "bolder", "lighter", "inherit",
        "100", "200", "300", "400", "500", "600", "700", "800", "900",
    ] = None


@dataclass
class Graphics(AttrsMixin):
    clip_rule: Literal["evenodd", "nonzero", "inherit"] | None = None
    cursor: None | Literal[
        "auto", "crosshair", "default", "pointer", "move",
        "e-resize", "ne-resize", "nw-resize", "n-resize", "se-resize", "sw-resize", "s-resize", "w-resize",
        "text", "wait", "help", "inherit",
    ] = None
    display: str | None = None
    filter: str | None = None
    pointer_events: None | Literal[
        "bounding-box", "visiblePainted", "visibleFill", "visibleStroke",
        "visible", "painted", "fill", "stroke", "all", "none",
    ] = None


@dataclass
class TextContentElements(AttrsMixin):
    direction: Literal["ltr", "rtl", "inherit"] | None = None
    dominant_baseline: None | Literal[
        "auto", "autosense-script", "no-change", "reset", "ideographic",
        "lower", "hanging", "mathematical", "inherit",
        "text-bottom", "alphabetic", "middle", "central", "text-top",
    ] = None
    letter_spacing: Literal["auto", "exact"] | None = None
    text_anchor: Literal["start", "middle", "end", "inherit"] | None = None
    text_decoration: None | Literal[
        "none", "underline", "overline", "line-through",
    ] = None
    unicode_bidi: None | Literal[
        "normal", "embed", "isolate", "bidi-override", "isolate-override", "plaintext",
    ] = None
    word_spacing: Literal["auto", "exact"] | None = None


@dataclass
class FilterPrimitive(AttrsMixin):
    x: Length | Number | None = None
    y: Length | Number | None = None


@dataclass
class ComponentTransferFunction(AttrsMixin):
    type: Literal["identity", "table", "discrete", "linear", "gamma"]
    tableValues: str | None = None
    intercept: float | None = None
    amplitude: float | None = None
    exponent: float | None = None
    offset: float | None = None


@dataclass
class Animation(AttrsMixin):
    # Animation value attributes
    calcMode: Literal["discrete", "linear", "paced", "spline"] | None = None
    values: str | list[Any] | None = None
    keyTimes: list[Number] | None = None
    keySplines: list[TimeBezierPoint] | None = None
    keyPoints: list[Number] | None = None
    from_: str | None = None
    to: str | None = None
    by: str | None = None

    # Animation target element attributes
    href: str | None = None

    # Animation addition attributes
    additive: Literal["replace", "sum"] | None = None
    accumulate: Literal["none", "sum"] | None = None


@dataclass
class AnimationTiming(AttrsMixin):
    begin: AnimationTimingEvent | list[AnimationTimingEvent] | None = None
    dur: timedelta | Literal["media", "indefinite"] | None = None
    end: AnimationTimingEvent | list[AnimationTimingEvent] | None = None
    min: timedelta | None = None
    max: timedelta | None = None
    restart: Literal["always", "never", "whenNotActive"] | None = None
    repeatCount: Number | Literal["indefinite"] | None = None
    repeatDur: timedelta | Literal["indefinite"] | None = None
    fill: Literal["freeze", "remove"] | None = None



from __future__ import annotations

from dataclasses import dataclass, fields

import svg


try:
    from functools import cached_property
except ImportError:
    cached_property = property  # type: ignore


@dataclass
class LibElement:
    title: str
    fields: list[str]

    @classmethod
    def parse_all(cls) -> list[LibElement]:
        result = []
        for el in svg.Element.__subclasses__():
            result.append(cls(
                title=el.element_name,
                fields=[f.name for f in fields(el)]
            ))
        return result

    @cached_property
    def attrs(self) -> set[str]:
        result = set()
        for field in self.fields:
            if field == 'elements':
                continue
            field = field.rstrip('_')
            field = field.replace('__', ':')
            field = field.replace('_', '-')
            result.add(field)
        return result



from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING, Any

from . import _mixins as m
from ._transforms import Transform
from ._types import Length, Number, PreserveAspectRatio
from .elements import Element


if TYPE_CHECKING:
    from typing_extensions import Literal


@dataclass
class Filter(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/filter
    """
    element_name = "filter"
    externalResourcesRequired: bool | None = None
    filterUnits: Literal["userSpaceOnUse", "objectBoundingBox"] | None = None
    primitiveUnits: Literal["userSpaceOnUse", "objectBoundingBox"] | None = None
    x: Length | Number | None = None
    y: Length | Number | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    class_: list[str] | None = None


@dataclass
class FeDistantLight(Element):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feDistantLight
    """
    element_name = "feDistantLight"
    azimuth: Number | None = None
    elevation: Number | None = None


@dataclass
class FePointLight(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/fePointLight
    """
    element_name = "fePointLight"
    x: Length | Number | None = None
    y: Length | Number | None = None
    z: Length | Number | None = None


@dataclass
class FeSpotLight(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feSpotLight
    """
    element_name = "feSpotLight"
    x: Length | Number | None = None
    y: Length | Number | None = None
    z: Length | Number | None = None
    pointsAtX: Number | None = None
    pointsAtY: Number | None = None
    pointsAtZ: Number | None = None
    specularExponent: Number | None = None
    limitingConeAngle: Number | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None


@dataclass
class FeBlend(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feBlend
    """
    element_name = "feBlend"
    in2: str | None = None
    mode: None | Literal[
        "normal", "multiply", "screen", "overlay", "darken", "lighten", "color-dodge",
        "color-burn", "hard-light", "soft-light", "difference", "exclusion", "hue",
        "saturation", "color", "luminosity", "inherit",
    ] = None
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeColorMatrix(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feColorMatrix
    """
    element_name = "feColorMatrix"
    type: Literal["matrix", "saturate", "hueRotate", "luminanceToAlpha"] | None = None
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    values: str | None = None


@dataclass
class FeComponentTransfer(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feComponentTransfer
    """
    element_name = "feComponentTransfer"
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeFuncR(Element, m.FilterPrimitive, m.ComponentTransferFunction):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feFuncR
    """
    element_name = "feFuncR"
    type2: Literal["identity", "table", "discrete", "linear", "gamma"] | None = None


@dataclass
class FeFuncG(Element, m.FilterPrimitive, m.ComponentTransferFunction):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feFuncG
    """
    element_name = "feFuncG"
    type2: Literal["identity", "table", "discrete", "linear", "gamma"] | None = None


@dataclass
class FeFuncB(Element, m.FilterPrimitive, m.ComponentTransferFunction):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feFuncB
    """
    element_name = "feFuncB"
    type2: Literal["identity", "table", "discrete", "linear", "gamma"] | None = None


@dataclass
class FeFuncA(Element, m.FilterPrimitive, m.ComponentTransferFunction):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feFuncA
    """
    element_name = "feFuncA"
    type3: Literal["identity", "table", "discrete", "linear", "gamma"] | None = None


@dataclass
class FeComposite(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feComposite
    """
    element_name = "feComposite"
    in2: str | None = None
    operator: Literal["over", "in", "out", "atop", "xor", "lighter", "arithmetic"] | None = None
    k1: Number | None = None
    k2: Number | None = None
    k3: Number | None = None
    k4: Number | None = None
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeConvolveMatrix(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feConvolveMatrix
    """
    element_name = "feConvolveMatrix"
    order: Any | None = None
    kernelMatrix: str | None = None
    divisor: Any | None = None
    bias: Any | None = None
    targetX: Any | None = None
    targetY: Any | None = None
    edgeMode: Literal["duplicate", "wrap", "none"] | None = None
    preserveAlpha: Any | None = None
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeDiffuseLighting(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feDiffuseLighting
    """
    element_name = "feDiffuseLighting"
    surfaceScale: Number | None = None
    diffuseConstant: Any | None = None
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    lighting_color: str | None = None


@dataclass
class FeDisplacementMap(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feDisplacementMap
    """
    element_name = "feDisplacementMap"
    in2: str | None = None
    scale: Any | None = None
    xChannelSelector: Any | None = None
    yChannelSelector: Any | None = None
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeFlood(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feFlood
    """
    element_name = "feFlood"
    flood_opacity: Number | None = None
    flood_color: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeGaussianBlur(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feGaussianBlur
    """
    element_name = "feGaussianBlur"
    stdDeviation: Number | tuple[Number, Number] | None = None
    edgeMode: Literal["duplicate", "wrap", "none"] | None = None
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeImage(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feImage
    """
    element_name = "feImage"
    externalResourcesRequired: Any | None = None
    transform: list[Transform] | None = None
    preserveAspectRatio: PreserveAspectRatio | None = None
    href: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeMerge(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feMerge
    """
    element_name = "feMerge"
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeMergeNode(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feMergeNode
    """
    element_name = "feMergeNode"
    in_: str | None = None


@dataclass
class FeMorphology(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feMorphology
    """
    element_name = "feMorphology"
    operator: Literal["erode", "dilate"] | None = None
    radius: Any | None = None
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeDropShadow(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feDropShadow
    """
    element_name = "feDropShadow"
    dx: Any | None = None
    dy: Any | None = None
    flood_opacity: Number | None = None
    flood_color: str | None = None
    stdDeviation: Number | tuple[Number, Number] | None = None
    class_: list[str] | None = None


@dataclass
class FeOffset(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feOffset
    """
    element_name = "feOffset"
    dx: Any | None = None
    dy: Any | None = None
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeSpecularLighting(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feSpecularLighting
    """
    element_name = "feSpecularLighting"
    surfaceScale: Number | None = None
    specularConstant: Number | None = None
    specularExponent: Number | None = None
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None
    lighting_color: str | None = None


@dataclass
class FeTile(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feTile
    """
    element_name = "feTile"
    in_: str | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None


@dataclass
class FeTurbulence(Element, m.FilterPrimitive):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Element/feTurbulence
    """
    element_name = "feTurbulence"
    baseFrequency: str | None = None
    numOctaves: int | None = None
    seed: Number | None = None
    stitchTiles: Literal["noStitch", "stitch"] | None = None
    type: Literal["fractalNoise", "turbulence"] | None = None
    color_interpolation_filters: Literal["auto", "sRGB", "linearRGB", "inherit"] | None = None
    result: str | None = None
    class_: list[str] | None = None
    width: Length | Number | None = None
    height: Length | Number | None = None



from __future__ import annotations

import re
from dataclasses import dataclass
from pathlib import Path

import yaml


try:
    from functools import cached_property
except ImportError:
    cached_property = property  # type: ignore


REX_EL = re.compile(r'li>\{\{\s*SVGElement\(["\']([a-zA-Z-]+)["\']\)\s*\}\}\.?</')
DEPRECATED = [
    # attributes of <font-face> which is deprecated
    'underline-thickness',
    'underline-position',
    'overline-thickness',
    'overline-position',
    'strikethrough-thickness',
    'strikethrough-position',

    # not supported by any browser
    'crossorigin',
    'systemLanguage',
]


@dataclass
class MDNAttr:
    title: str
    slug: str
    content: str
    status: list[str] | None = None
    spec_urls: list[str] | None = None
    browser_compat: object | None = None
    sidebar: object | None = None

    @classmethod
    def parse(cls, path: Path) -> MDNAttr | None:
        path /= 'index.md'
        if not path.is_file():
            return None
        raw = path.read_text()
        front_matter, _, markdown = raw.partition('\n---\n')
        fields: dict = next(yaml.load_all(front_matter, Loader=yaml.SafeLoader))
        page_type = fields.pop('page-type')
        assert page_type == 'svg-attribute'
        fields = {k.replace('-', '_'): v for k, v in fields.items()}
        return cls(**fields, content=markdown)

    @classmethod
    def parse_all(cls, path: Path) -> list[MDNAttr]:
        result = []
        for subpath in (path / 'attribute').iterdir():
            attr = cls.parse(subpath)
            if attr is not None:
                result.append(attr)
        return result

    @cached_property
    def is_deprecated(self) -> bool:
        if self.title in DEPRECATED:
            return True
        if self.status and 'deprecated' in self.status:
            return True
        if '{{deprecated_header}}' in self.content.lower():
            return True
        return False

    @cached_property
    def elements(self) -> set[str]:
        sep = 'You can use this attribute with the following SVG elements:'
        _, _, txt = self.content.partition(sep)
        txt, _, _ = self.content.partition('</ul>')
        txt, _, _ = self.content.partition('See also')
        result = set()
        for match in REX_EL.finditer(txt):
            result.add(match.group(1))
        return result



from . import animate, animate_transform, grid3, grid4, shapes, text, transform


__all__ = [
    'animate',
    'animate_transform',
    'grid3',
    'grid4',
    'shapes',
    'text',
    'transform',
]



from __future__ import annotations

from dataclasses import dataclass

from ._types import Number


class Transform:
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/transform
    """
    pass


@dataclass
class Matrix(Transform):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/transform#matrix
    """
    a: Number
    b: Number
    c: Number
    d: Number
    e: Number
    f: Number

    def __str__(self):
        return f'matrix({self.a} {self.b} {self.c} {self.d} {self.e} {self.f})'


@dataclass
class Translate(Transform):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/transform#translate
    """
    x: Number
    y: Number | None = None

    def __str__(self):
        if self.y is None:
            return f'translate({self.x})'
        return f'translate({self.x} {self.y})'


@dataclass
class Scale(Transform):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/transform#scale
    """
    x: Number
    y: Number | None = None

    def __str__(self):
        if self.y is None:
            return f'scale({self.x})'
        return f'scale({self.x} {self.y})'


@dataclass
class Rotate(Transform):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/transform#rotate
    """
    a: Number
    x: Number | None = None
    y: Number | None = None

    def __str__(self):
        if self.x is None:
            return f'rotate({self.a})'
        assert self.y is not None
        return f'rotate({self.a} {self.x} {self.y})'


@dataclass
class SkewX(Transform):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/transform#skewx
    """
    a: Number

    def __str__(self):
        return f'skewX({self.a})'


@dataclass
class SkewY(Transform):
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/transform#skewy
    """
    a: Number

    def __str__(self):
        return f'skewY({self.a})'



"""
Tutorial:
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/transform

Usage:
    PYTHONPATH=. python3 examples/transform.py > examples/transform.svg
    xdg-open examples/transform.svg

We use functions instead of <g> and <use>
because it gives a better control over types.
"""
from __future__ import annotations

import svg


def heart_path() -> list[svg.PathData]:
    return [
        svg.M(10, 30),
        svg.Arc(20, 20, 0, False, True, x=50, y=30),
        svg.Arc(20, 20, 0, False, True, x=90, y=30),
        svg.Q(90, 60, 50, 90),
        svg.Q(10, 60, 10, 30),
        svg.Z(),
    ]


def draw() -> svg.SVG:
    return svg.SVG(
        viewBox=svg.ViewBoxSpec(-40, 0, 150, 100),
        elements=[
            svg.Path(
                d=heart_path(),
                fill="grey",
                transform=[
                    svg.Rotate(-10, 50, 100),
                    svg.Translate(-36, 45.5),
                    svg.SkewX(40),
                    svg.Scale(1, 0.5),
                ],
            ),
            svg.Path(
                d=heart_path(),
                fill="none",
                stroke="red",
            ),
        ],
    )


if __name__ == '__main__':
    print(draw())



from __future__ import annotations
from datetime import datetime

import math
from dataclasses import dataclass
from datetime import timedelta
from decimal import Decimal

from typing import TYPE_CHECKING, Union


if TYPE_CHECKING:
    from typing_extensions import Literal


Number = Union[Decimal, float, int]


@dataclass
class Length:
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Content_type#length
    """

    value: Number
    unit: Literal["em", "ex", "px", "pt", "pc", "cm", "mm", "in", "%"]

    def __str__(self) -> str:
        return f"{self.value}{self.unit}"


@dataclass
class PreserveAspectRatio:
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/preserveAspectRatio
    """

    alignment: Literal[
        "none",
        "xMinYMin",
        "xMidYMin",
        "xMaxYMin",
        "xMinYMid",
        "xMidYMid",
        "xMaxYMid",
        "xMinYMax",
        "xMidYMax",
        "xMaxYMax",
    ] = "xMidYMid"
    scale_type: Literal["meet", "slice"] = "meet"

    def __str__(self) -> str:
        return f"{self.alignment} {self.scale_type}"


@dataclass
class ViewBoxSpec:
    """
    https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/viewBox
    """

    min_x: Number
    min_y: Number
    width: Number
    height: Number

    def __str__(self) -> str:
        return f"{self.min_x} {self.min_y} {self.width} {self.height}"


@dataclass
class TimeBezierPoint:
    x1: Number
    y1: Number
    x2: Number
    y2: Number

    def __post_init__(self) -> None:
        assert self.x1 >= 0 and self.x1 <= 1
        assert self.y1 >= 0 and self.y1 <= 1
        assert self.x2 >= 0 and self.x2 <= 1
        assert self.y2 >= 0 and self.y2 <= 1

    def __str__(self) -> str:
        return f"{self.x1} {self.y1} {self.x2} {self.y2}"


# https://developer.mozilla.org/en-US/docs/Web/SVG/Reference/Attribute/begin
# https://developer.mozilla.org/en-US/docs/Web/SVG/Reference/Attribute/end

@dataclass
class SyncbaseValue:
    element_id: str
    event: Literal["begin", "end"]
    offset: timedelta | None

    def __str__(self) -> str:
        str_value = f"{self.element_id}.{self.event}"
        if self.offset is not None:
            offset_str = to_clock_value(self.offset)
            if not offset_str.startswith("-"):
                offset_str = "+" + offset_str
            str_value += offset_str
        return str_value


@dataclass
class EventValue:
    element_id: str | None
    event: Literal[
        "focus",
        "blur",
        "focusin",
        "focusout",
        "DOMActivate",
        "auxclick",
        "click",
        "dblclick",
        "mousedown",
        "mouseenter",
        "mouseleave",
        "mousemove",
        "mouseout",
        "mouseover",
        "mouseup",
        "wheel",
        "beforeinput",
        "input",
        "keydown",
        "keyup",
        "compositionstart",
        "compositionupdate",
        "compositionend",
        "load",
        "unload",
        "abort",
        "error",
        "select",
        "resize",
        "scroll",
        "beginEvent",
        "endEvent",
        "repeatEvent",
    ]
    offset: timedelta | None

    def __str__(self) -> str:
        str_value = ""
        if self.element_id is not None:
            str_value= f"{self.element_id}."
        str_value += f"{self.event}"
        if self.offset is not None:
            offset_str = to_clock_value(self.offset)
            if not offset_str.startswith("-"):
                offset_str = "+" + offset_str
            str_value += offset_str
        return str_value


@dataclass
class RepeatValue:
    element_id: str | None
    repeat_number: int
    offset: timedelta | None

    def __str__(self) -> str:
        str_value = ""
        if self.element_id is not None:
            str_value = f"{self.element_id}."
        str_value+= f"repeat({self.repeat_number})"

        if self.offset is not None:
            offset_str = to_clock_value(self.offset)
            if not offset_str.startswith("-"):
                offset_str = "+" + offset_str
            str_value += offset_str
        return str_value


@dataclass
class AccessKeyValue:
    key: str
    offset: timedelta | None

    def __str__(self) -> str:
        str_value = f"accessKey({self.key})"
        if self.offset is not None:
            offset_str = to_clock_value(self.offset)
            if not offset_str.startswith("-"):
                offset_str = "+" + offset_str
            str_value += offset_str
        return str_value


AnimationTimingEvent = Union[
    timedelta,
    SyncbaseValue,
    EventValue,
    RepeatValue,
    AccessKeyValue,
    datetime,
    'Literal["indefinite"]'
]


def to_clock_value(delta: timedelta) -> str:
    """Format timedelta as ClockValue SVG type.

    https://developer.mozilla.org/en-US/docs/Web/SVG/Guides/Content_type#clock-value
    https://svgwg.org/specs/animations/#ClockValueSyntax
    """
    seconds = delta.total_seconds()

    sign = ""
    if seconds < 0:
        sign = "-"
        seconds = abs(seconds)

    partial_seconds = seconds - math.floor(seconds)
    seconds = int(seconds)
    fraction = ""
    if partial_seconds > 0:
        fraction = f"{partial_seconds:.6f}".strip("0")

    # Format as Timecount-val.
    if abs(seconds) < 60:
        # The "s" suffix is optional but it's good for readability:
        # in JS, time is represented in miliseconds, so
        # just a number without a suffix may confuse JS engineers.
        return f"{sign}{seconds}{fraction}s"

    # Format as Full-clock-val.
    minutes, full_seconds = divmod(seconds, 60)
    hours, minutes = divmod(minutes, 60)
    return f"{sign}{hours}:{minutes:02}:{full_seconds:02}{fraction}"


def to_wallclock_sync_value(time: datetime) -> str:
    iso_time = time.isoformat(" ")
    return f"wallclock({iso_time})"
